//  ************************************************************************************************
//
//  BornAgain: simulate and fit reflection and scattering
//
//! @file      GUI/View/List/SamplesQModel.cpp
//! @brief     Implements class SamplesQModel.
//!
//! @homepage  http://www.bornagainproject.org
//! @license   GNU General Public License v3 or higher (see COPYING)
//! @copyright Forschungszentrum Jülich GmbH 2021
//! @authors   Scientific Computing Group at MLZ (see CITATION, AUTHORS)
//
//  ************************************************************************************************

#include "GUI/View/List/SamplesQModel.h"
#include "Base/Util/Assert.h"
#include "GUI/Model/FromCore/GUIExamplesFactory.h"
#include "GUI/Model/FromCore/ItemizeSample.h"
#include "GUI/Model/Project/ProjectDocument.h"
#include "GUI/Model/Sample/SampleItem.h"
#include "GUI/Model/Sample/SamplesSet.h"
#include "GUI/Support/Util/String.h"
#include "GUI/View/Manager/PyImportAssistant.h"
#include "Sample/Multilayer/MultiLayer.h"
#include <QColor>

SamplesQModel::SamplesQModel(QObject* parent)
    : QAbstractListModel(parent) // parent needed to ensure end of life
{
}

SamplesSet* SamplesQModel::set()
{
    return gDoc->samplesModifier();
}

const SamplesSet* SamplesQModel::set() const
{
    return gDoc->samples();
}

int SamplesQModel::rowCount(const QModelIndex&) const
{
    return set()->size();
}

QVariant SamplesQModel::data(const QModelIndex& index, int role) const
{
    if (!index.isValid())
        return {};
    size_t row = index.row();
    if (row >= set()->size())
        return {};
    const SampleItem* t = set()->at(row);

    switch (role) {
    case Qt::DisplayRole:
        return t->name();
    case Qt::ToolTipRole:
        return t->description();
    case Qt::BackgroundRole:
        return row == set()->currentIndex() ? QColor(Qt::green) : QColor(Qt::magenta);
    default:
        return {};
    }
}

const SampleItem* SamplesQModel::itemForIndex(const QModelIndex& index) const
{
    if (!index.isValid())
        return nullptr;
    return set()->at(index.row());
}

SampleItem* SamplesQModel::itemForIndex(const QModelIndex& index)
{
    if (!index.isValid())
        return nullptr;
    return set()->at(index.row());
}

QModelIndex SamplesQModel::indexForItem(SampleItem* item) const
{
    if (auto row = set()->index_of(item); row >= 0)
        return index(row, 0);
    return {};
}

void SamplesQModel::removeSample(SampleItem* item)
{
    QModelIndex index = indexForItem(item);
    if (!index.isValid())
        return;

    beginRemoveRows(index.parent(), index.row(), index.row());
    set()->delete_element(item);
    endRemoveRows();
}

QModelIndex SamplesQModel::copySample(SampleItem* item)
{
    ASSERT(item);
    const QStringList existingNames = set()->sampleNames();

    const int row = set()->size();
    beginInsertRows(QModelIndex(), row, row);
    SampleItem* copied_item = item->clone();
    copied_item->setName(GUI::Util::String::suggestName(existingNames, item->name()));
    set()->push_back(copied_item);
    endInsertRows();
    return indexForItem(copied_item);
}

QModelIndex SamplesQModel::createSample()
{
    const QStringList existingNames = set()->sampleNames();

    auto* t = new SampleItem;
    t->setName(GUI::Util::String::suggestName(existingNames, "Sample"));
    t->addStandardMaterials();

    const int row = set()->size();

    beginInsertRows(QModelIndex(), row, row);
    set()->push_back(t);
    endInsertRows();

    return indexForItem(t);
}

QModelIndex SamplesQModel::createSampleFromExamples(const QString& className, const QString& title,
                                                    const QString& description)
{
    auto* t = dynamic_cast<SampleItem*>(GUI::ExamplesFactory::itemizeSample(className));
    if (!t)
        return {};
    t->setName(title);
    t->setDescription(description);

    const int row = set()->size();

    beginInsertRows(QModelIndex(), row, row);
    set()->push_back(t);
    endInsertRows();

    return indexForItem(t);
}

#ifdef BORNAGAIN_PYTHON
QModelIndex SamplesQModel::createSampleFromPython()
{
    std::unique_ptr<MultiLayer> sample = PyImportAssistant::importMultiLayer();
    if (!sample)
        return {}; // any messages already shown to user

    SampleItem* t = PyImportAssistant::itemizeSample(*sample);
    if (!t)
        return {}; // any messages already shown to user
    t->setDescription("Imported from python code");

    const int row = set()->size();

    beginInsertRows(QModelIndex(), row, row);
    set()->push_back(t);
    endInsertRows();

    return indexForItem(t);
}
#endif
